Package: mbQTL
Type: Package
Title: mbQTL: A package for SNP-Taxa mGWAS analysis
Version: 1.13.0
Authors@R: 
    c(person(given = "Mercedeh",
           family = "Movassagh",
           role = c("aut", "cre"),
           email = "mercedeh.movassagh@yale.edu",
           comment = c(ORCID = "0000-0001-7690-0230")),
           person("Steven","Schiff", role="aut"),
           person("Joseph N", "Paulson", role="aut"))
Description: mbQTL is a statistical R package for simultaneous 16srRNA,16srDNA (microbial)
 and variant, SNP, SNV (host) relationship, correlation, regression studies.
 We apply linear, logistic and correlation based statistics to identify the relationships
 of taxa, genus, species and variant, SNP, SNV in the infected host. We produce various 
 statistical significance measures such as P values, FDR, BC and probability estimation to
 show significance of these relationships. Further we provide various visualization function
 for ease and clarification of the results of these analysis. The package is compatible with
 dataframe, MRexperiment and text formats.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.3.0)
DeploySubPath: mbQTL
biocViews: SNP, Microbiome, WholeGenome, Metagenomics,
        StatisticalMethod, Regression
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.1
Imports: MatrixEQTL, dplyr, ggplot2, readxl, stringr, tidyr,
        metagenomeSeq, pheatmap, broom, graphics, stats, methods
Suggests: knitr, rmarkdown, BiocStyle
VignetteBuilder: knitr
URL: "https://github.com/Mercedeh66/mbQTL"
BugReport: "https://github.com/Mercedeh66/mbQTL/issues"
git_url: https://git.bioconductor.org/packages/mbQTL
git_branch: devel
git_last_commit: dcb1e5a
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 22:35:45 UTC; biocbuild
Author: Mercedeh Movassagh [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7690-0230>),
  Steven Schiff [aut],
  Joseph N Paulson [aut]
Maintainer: Mercedeh Movassagh <mercedeh.movassagh@yale.edu>
Built: R 4.6.0; ; 2026-05-19 04:57:32 UTC; unix

# Generated by roxygen2: do not edit by hand

export(allToAllProduct)
export(coringTaxa)
export(histPvalueLm)
export(linearTaxaSnp)
export(logRegSnpsTaxa)
export(logitPlotSnpTaxa)
export(mbQtlCorHeatmap)
export(metagenomeSeqToMbqtl)
export(qqPlotLm)
export(taxaSnpCor)
import(MatrixEQTL)
import(broom)
import(dplyr)
import(ggplot2)
import(metagenomeSeq)
import(pheatmap)
import(readxl)
import(stringr)
import(tidyr)
importFrom(graphics,hist)
importFrom(methods,is)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,glm)
importFrom(stats,p.adjust)
importFrom(stats,quantile)

#<img src="man/figures/logo.png" align="right" alt="logo.png" width="180" />

# mbQTL

<!-- badges: start -->
[![BioC status](http://www.bioconductor.org/shields/build/release/bioc/mbQTL.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/mbQTL)
<!-- badges: end -->

## Installation
```
devtools::install_github('Mercedeh66/mbQTL')

if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("mbQTL")
```
## Issues and bug reports
Please use https://github.com/Mercedeh66/mbQTL/issues to submit issues, bug reports, and comments.

#The microbiomeAbund file is a randomly generated file format for total microbe presence (number of reads)
#(parasite/viral transcripts) for specific species. This could be generated from select taxa results from biom() or
#MRexperiment objects as long as the samples are colnames and taxa are rownames.
microbeAbund <- read.table("data-raw/Taxa_table_MVP_cor.txt", sep = "\t", row.names = 1, header = TRUE)
#The SNP file is randomly generated from GATK (Van der Auwera GA & O'Connor BD. (2020). Genomics in the Cloud:
#Using Docker, GATK, and WDL in Terra (1st Edition). O'Reilly Media) snp calls followed by plink (Purcell S, et al. (2007)
#PLINK: a toolset for whole-genome association and population-based linkage analysis. American Journal of Human Genetics)
# processing and it needs to be in (0,1,2) format representing the zygosity of the snps.
SnpFile <- read.table("data-raw/SNP_table_MVP_cor.txt", sep = "\t", row.names = 1, header = TRUE)
#The covariance file is generated randomly by assigning sex and site of collection to the samples.)
#rownames are covariate and colnames samples.
CovFile <- read.table("data-raw/covariates.txt", sep = "\t", row.names = 1, header = TRUE)
#MetagenomSeqObj is an `MRexperiment` object format of the "microbeAbund" file.
metagenomeSeqObj <- readRDS("data-raw/metagenomeSeqReads.RDS")
usethis::use_data(microbeAbund, SnpFile, CovFile, metagenomeSeqObj, overwrite = TRUE)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----style, echo=FALSE, results='asis'----------------------------------------
BiocStyle::markdown()

## ----setup--------------------------------------------------------------------
knitr::opts_chunk$set(
  dpi = 300,
  warning = FALSE,
  collapse = TRUE,
  error = FALSE,
  comment = "#>",
  echo = TRUE
)
library(mbQTL)
library(tidyr)

data("SnpFile")
data("microbeAbund")
data("CovFile")
data("metagenomeSeqObj")

doctype <- knitr::opts_knit$get("rmarkdown.pandoc.to")

## -----------------------------------------------------------------------------
sessionInfo()

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
# microbial count file
# microbeAbund
# SNP file
# SnpFile
# Covariance file
# CovFile

#Check out file formats compatible with mbQTL files.

#head(microbeAbund)
#head(SnpFile)
#head(CovFile)

metagenomeSeqObj


## ----eval=TRUE, echo=TRUE-----------------------------------------------------
compatible_metagenome_seq <- metagenomeSeqToMbqtl(metagenomeSeqObj,
  norm = TRUE, log = TRUE,
  aggregate_taxa = "Genus"
)

# Check for the class of compatible_metagenome_seq

class(compatible_metagenome_seq)


## ----eval=TRUE, echo=TRUE-----------------------------------------------------

# perform linear regression analysis between taxa and snps across 
# samples and produce a data frame of results.(covariate file is
# optional but highly recommended to avoid results which might be
# prone to batch effects and obtaining optimal biological relevance
# for finding snp - taxa relationships.) 

LinearAnalysisTaxaSNP <- linearTaxaSnp(microbeAbund,
  SnpFile,
  Covariate = CovFile
)

## ----eval=TRUE, echo=TRUE, fig.width = 4, fig.height=4, out.width="90%", dpi=300, fig.align="center"----

# Create a histogram of P values across all snp - taxa
# linear regression estimations

histPvalueLm(LinearAnalysisTaxaSNP)


# Create a QQPlot of expected versus estimated P value for all all
# snp - taxa linear regression estimations

qqPlotLm(microbeAbund, SnpFile, Covariate = CovFile)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------

# Estimate taxa SNP - taxa correlation R and estimate R2
# (To what extent variance in one taxa explains the
# variance in snp)

# First estimate R value for all SNP-Taxa relationships

correlationMicrobes <- coringTaxa(microbeAbund)

# Estimate the correlation value (R2) between a specific snp and all taxa.

one_rs_id <- allToAllProduct(SnpFile, microbeAbund, "chr1.171282963_T")

# Estimate the correlation value (R2) between all snps and all taxa.

for_all_rsids <- allToAllProduct(SnpFile, microbeAbund)

# Estimate rho value for all snps and all taxa present in the dataset.

taxa_SNP_Cor <- taxaSnpCor(for_all_rsids, correlationMicrobes)

# Estimate rho value for all snps and all taxa present in the
# dataset and pick the highest and lowest p values in the
# dataset to identify improtant SNP-Taxa relationships.

taxa_SNP_Cor_lim <- taxaSnpCor(for_all_rsids,
  correlationMicrobes,
  probs = c(0.0001, 0.9999)
)

## ----eval=TRUE, echo=TRUE, fig.width = 9, fig.height=5, out.width="90%", dpi=300, fig.align="center"----

## Draw heatmap of rho estimates "taxa_SNP_Cor_lim" is the
# output of taxaSnpCor().
# One can use other pheatmap() settings for extra annotation

mbQtlCorHeatmap(taxa_SNP_Cor_lim,
  fontsize_col = 5,
  fontsize_row = 7
)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------

# perform Logistic regression analysis between taxa and snps 
# across samples microbeAbund is the microbe abundance file
# (a dateframe with rownames as taxa and colnames and sample
# names) and SnpFile is the snp file (0,1,2) values
# (rownames as snps and colnames as samples)

log_link_resA <- logRegSnpsTaxa(microbeAbund, SnpFile)

# Perform Logistic regression for specific microbe

log_link_resB <- logRegSnpsTaxa(microbeAbund, SnpFile,
  selectmicrobe = c("Haemophilus")
)

## ----eval=TRUE, echo=TRUE, fig.width = 5, fig.height=2, out.width="90%", dpi=300, fig.align="center"----

# Create a barplot with the specific rsID, and microbe of interest,
# including the genotype information for the reference versus 
# alternate versus hetrozygous alleles and and presence absence
# of microbe of interest. Note your reference, alternative and
# heterozygous genotype values need to match the genotype of
# your SNP of interest this information can be obtained
# from snpdb data base or GATK/plink files.

Logit_plot <- logitPlotSnpTaxa(microbeAbund, SnpFile,
  selectmicrobe = "Neisseria",
  rsID = "chr2.241072116_A", ref = "GG", alt = "AA", het = "AG"
)

Logit_plot

#' @import ggplot2
#' @import readxl
#' @import tidyr
#' @import dplyr
#' @import stringr
#' @importFrom stats as.formula glm p.adjust
#' @importFrom methods is
NULL

if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    ".", "term", "p.value", "Pathogen", "genotype"
  ))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  `binarizeMicrobe` binarizes microbe abundace file based on user's cutoff
#'
#' This function creates a dataframe output produces a formatted dataframe prepared.
#' @param microbeAbund the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)
#' @param cutoff cutoff at which the user chose to call taxa positive or negative across samples (should be a numeric value
#' for normalized or count values).
#' @param selectmicrobe default is and all taxa are considered at the same time, if the user is interested in a
#' specific pathogen use name of the pathogen for example "Haemophilus".
#' @return A data frame of microbial abundance.
#' @keywords logitPlotDataframe logitParsing
#'
binarizeMicrobe <- function(microbeAbund, cutoff = NULL, selectmicrobe = NULL) {
  if (is.null(cutoff)) { # If value of rsID is not null do this:
    micrbial_abundance_2 <- microbeAbund %>% mutate_if(is.numeric, ~ 1 * (. != 0))
  } else {
    micrbial_abundance_2 <- microbeAbund %>% mutate_if(is.numeric, ~ 1 * (. > cutoff))
  }
  if (is.null(selectmicrobe)) { # If value of rsID is not null do this:
    micrbial_abundance_3 <- micrbial_abundance_2
    micrbial_abundance_3$ID <- rownames(micrbial_abundance_3)
    return(micrbial_abundance_3)
  } else {
    micrbial_abundance_3 <- micrbial_abundance_2 %>% select(all_of(selectmicrobe))
    micrbial_abundance_3$ID <- rownames(micrbial_abundance_3)
    return(micrbial_abundance_3)
  }
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  `prepareCorData` prpares and joins snp-taxa and taxa-taxa correlation file.
#'
#' This function creates a dataframe output produces a formatted dataframe prepared.
#' @param microbeAbund the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)
#' @param SnpFile the snp dataframe (values 0,1,2 indicating zygosity), rownames sample names and colnames snp names.
#' @param cutoff default is NULL, hence anything above cutoff is considered positive, the cutoff at which the
#' specific or all taxa are considered positive for the pathogen (1 indicates positive and 0 negative).
#' @param selectmicrobe default is and all taxa are considered at the same time, if the user is interested in a
#' specific pathogen use name of the pathogen for example "Haemophilus".
#' @return A data frame which is a result of Logistic regression  products of individual snp, taxa relationships,
#' with P values and P value corrected values.
#' @keywords logitPlotDataframe
#'
## Function to produce dataframe for plot
prepareCorData <- function(microbeAbund, SnpFile, cutoff = NULL, selectmicrobe = NULL) {
  # Function binarizes based on user cutoff and either select microbe or look across all
  microbial_abundance_4 <- binarizeMicrobe(microbeAbund, cutoff = cutoff, selectmicrobe = selectmicrobe)
  n_snps <- ncol(SnpFile)
  SnpFile$ID <- rownames(SnpFile)
  SNPAss_filt <- SnpFile %>%
    pivot_longer(seq.int(1, n_snps), names_to = "rsID", values_to = "count")
  final_DF_Logit <- left_join(SNPAss_filt, microbial_abundance_4, by = "ID")
  return(final_DF_Logit)
}


## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  `logRegSnpsTaxa` Performs logistic regression analysis between taxa and SNPs and returns concordance statistics
#'
#' This function creates a dataframe output from the results of either a unique taxa and all snps or all taxa and
#' all snps in the dataset. The result is a dataframe with P values and FDRs of all regressions.
#' @param microbeAbund the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)
#' @param SnpFile the snp dataframe (values 0,1,2 indicating zygosity), rownames sample names and colnames snp names.
#' @param cutoff default is NULL, hence anything above cutoff is considered positive, the cutoff at which the
#' specific or all taxa are considered positive for the pathogen (1 indicates positive and 0 negative).
#' @param selectmicrobe default is and all taxa are considered at the same time, if the user is interested in a
#' specific pathogen use name of the pathogen for example "Haemophilus".
#' @return A data frame which is a result of Logistic regression  products of individual snp, taxa relationships,
#' with P values and P value corrected values (FDR, Bonferroni).
#' @export
#' @keywords taxa snp logistic_regression logit
#' @examples
#' data(microbeAbund)
#' data(SnpFile)
#' x <- logRegSnpsTaxa(microbeAbund, SnpFile, selectmicrobe = c("Haemophilus"))
#'
logRegSnpsTaxa <- function(microbeAbund, SnpFile, cutoff = NULL, selectmicrobe = NULL) {
  stopifnot("data frame is the expected input for SnpFile" = is(SnpFile, "data.frame"))
  stopifnot("data frame is the expected input for microbeAbund" = is(microbeAbund, "data.frame"))
  stopifnot("The value for 'cutoff'" = is.null(cutoff) || is.numeric(cutoff))
  stopifnot("The value for 'selectmicrobe'" = is.null(selectmicrobe) || is.character(selectmicrobe))
  # Function binarizes based on user cutoff and either select microbe or look across all
  final_DF_Logit <- prepareCorData(microbeAbund, SnpFile,
    cutoff = cutoff,
    selectmicrobe = selectmicrobe
  )

  # if (is.null(selectmicrobe)){
  non_microb <- c("ID", "rsID", "count")
  microbes <- setdiff(colnames(final_DF_Logit), non_microb)
  chelo <- lapply(
    microbes,
    function(m) {
      x <- final_DF_Logit %>% group_by(rsID)
      ret <- do(x, tidy(glm(as.formula(paste(m, "~ count")), family = "binomial", data = .)))
      ret$microbe <- m
      return(ret)
    }
  ) %>%
    bind_rows()
  chelo_play <- filter(chelo, term == c("count"))
  # Add FDR coloumn or Bonf to the col
  chelo_play$FDR <- p.adjust(chelo_play$p.value, method = "fdr")
  chelo_play$BC <- p.adjust(chelo_play$p.value, method = "BH")
  # Sort by P values
  chelo_playF <- chelo_play %>% arrange(p.value)
  chelo_playFin <- chelo_playF %>% select(-term)
  return(chelo_playFin)
}


# log_link_res<-logRegSnpsTaxa(microbeAbund,SnpFile)
# log_link_res<-logRegSnpsTaxa(microbeAbund,SnpFile,selectmicrobe = c("Haemophilus"))


## Plot function for Logistic Regression
## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  logitPlotSnpTaxa produces bar plots for counts of ref vs alt vs het allells for
#'  particular rsID taxa combinations
#'
#' This function creates a dataframe output produces a formatted dataframe prepared.
#' @param SnpFile original snp file with (0,1,2 values for ref, het, alt genotypes), colnames
#'                SNP names, rownames, sample IDs.
#' @param microbeAbund original microbe abundance file (colnames microbe, rownames= sample IDs)
#' @param selectmicrobe name of the microbe of interest (for example individual significant microbes
#'                      associate with a snp).
#' @param rsID name of the snp of interest (for example individual significant snps associated with
#'             a microbe)
#' @param ref the name of reference genotype for example "GG"
#' @param alt the name of snp (variant) genotype for example "AA"
#' @param het the name of hetrozygote genotype for example "GA"
#' @param color the default is NULL and the color is set to c("#ffaa1e", "#87365b").
#' @param cutoff cutoff at which we call microbe present or absent
#' @return A bar ggplot comparing the counts of ref vs alt vs het genotype
#' @export
#' @keywords barplot logitplot
#' @examples
#' data(microbeAbund)
#' data(SnpFile)
#' x <- logitPlotSnpTaxa(microbeAbund, SnpFile,
#'   selectmicrobe = "Neisseria", rsID = "chr2.241072116_A",
#'   ref = NULL, alt = NULL, het = NULL, color = NULL, cutoff = NULL
#' )
logitPlotSnpTaxa <-
  function(microbeAbund, SnpFile, selectmicrobe = NULL,
           rsID, ref = NULL, alt = NULL, het = NULL, color = NULL, cutoff = NULL) {
    stopifnot("data frame is the expected input for SnpFile" = is(SnpFile, "data.frame"))
    stopifnot("data frame is the expected input for microbeAbund" = is(microbeAbund, "data.frame"))
    stopifnot("The value for 'rsID'" = is.null(cutoff) || is.character(rsID))
    stopifnot("The value for 'selectmicrobe'" = is.null(cutoff) || is.character(selectmicrobe))
    stopifnot("The value for 'ref'" = is.null(ref) || is.character(ref))
    stopifnot("The value for 'alt'" = is.null(alt) || is.character(alt))
    stopifnot("The value for 'het'" = is.null(het) || is.character(het))
    stopifnot("The value for 'color'" = is.null(color) || is.character(color))
    stopifnot("The value for 'cutoff'" = is.null(cutoff) || is.numeric(cutoff))
    final_DF_Logit <- prepareCorData(microbeAbund, SnpFile,
      cutoff = cutoff,
      selectmicrobe = selectmicrobe
    )

    final_DF_Logit_P <- as.data.frame(final_DF_Logit)
    final_DF_Logit_P_rs <- final_DF_Logit_P %>% filter(rsID == !!rsID)
    final_DF_Logit_plot <- final_DF_Logit_P_rs %>% select(all_of(c("ID", "count", "rsID", selectmicrobe)))

    # Ensure relevant columns are factors
    for (colname in c("count", "rsID", selectmicrobe)) {
      final_DF_Logit_plot[[colname]] <- as.factor(final_DF_Logit_plot[[colname]])
    }

    if (!is.null(ref)) {
      final_DF_Logit_plot_cased <- final_DF_Logit_plot %>%
        mutate(genotype = case_when(
          count == 2 ~ alt,
          count == 0 ~ ref,
          count == 1 ~ het
        ))
    } else {
      final_DF_Logit_plot_cased <- final_DF_Logit_plot %>%
        mutate(genotype = case_when(
          count == 2 ~ "2",
          count == 0 ~ "0",
          count == 1 ~ "1"
        ))
    }
    count_final_DF_Logit_plot_cased2 <-
      final_DF_Logit_plot_cased %>%
      mutate(Pathogen = case_when(
        final_DF_Logit_plot_cased[[selectmicrobe]] == 0 ~ "Absent",
        final_DF_Logit_plot_cased[[selectmicrobe]] == 1 ~ "Present"
      ))
    count_plot_file <- count_final_DF_Logit_plot_cased2 %>% count(Pathogen, genotype)
    if (is.null(color)) {
      # If value of rsID is not null do this:
      p <-
        ggplot(
          count_plot_file,
          aes(x = genotype, y = n, fill = Pathogen)
        ) +
        geom_bar(position = "dodge", stat = "identity") +
        scale_fill_manual(values = c("#ffaa1e", "#87365b")) +
        ylab("Genotype Count") +
        xlab(paste(rsID, "count"))
      return(p)
    } else {
      p <-
        ggplot(
          count_plot_file,
          aes(x = genotype, y = n, fill = Pathogen)
        ) +
        geom_bar(position = "dodge", stat = "identity") +
        scale_fill_manual(values = color) +
        ylab("Genotype Count") +
        xlab(paste(rsID, "count"))
      return(p)
    }
  }


# logit_plot <- logitPlotSnpTaxa(microbeAbund, SnpFile, selectmicrobe = "Neisseria",
#                                 rsID="chr2.241072116_A", ref="GG", alt="AA",het="AG")

#' @import ggplot2
#' @import readxl
#' @import tidyr
#' @import dplyr
#' @import stringr
#' @import pheatmap
#' @import broom
#' @importFrom stats cor quantile
#' @importFrom methods is
NULL

# un-upset lintr
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    "Genus_1", "Genus_2", "product_r2", "R", "Genus1", "Genus2", "X1", "X2", "rsID",
    "rho_xy", "Combined_Genus"
  ))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  `coringTaxa` creates correlation dataframe for taxa
#'
#' This function creates an output correlation data frame for all microbial taxa (or other organisms such as
#' viral or parasitic taxa)
#' @param microbeAbund the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)
#' @return A data frame of correlations between taxa
#' @export
#' @keywords taxa correlation
#' @examples
#' data(microbeAbund)
#' x <- coringTaxa(microbeAbund)
#'
# Function to run correlation on the taxa-taxa
coringTaxa <- function(microbeAbund) {
  stopifnot("data frame is the expected input for microbeAbund" = is(microbeAbund, "data.frame"))
  # start parsing
  ppr <- rownames(microbeAbund)
  vapply_type <- array(data = 0.0, dim = nrow(microbeAbund))
  microbeAbundA <- as.data.frame(vapply(microbeAbund, as.numeric, vapply_type))
  rownames(microbeAbundA) <- ppr
  coringTaxaR <- cor(microbeAbundA)
  coringTaxaF <- as.data.frame(coringTaxaR)
  # Make a long R dataframe
  coringTaxaF$Genus_1 <- rownames(coringTaxaF)
  coringTaxalong <- coringTaxaF %>%
    pivot_longer(cols = 0:length(rownames(coringTaxaF)), names_to = "Genus_2", values_to = "R")
  # Remove genus correlations which are the same for Genus_1 and 2 values
  coringTaxalongParsed <- coringTaxalong %>% filter(Genus_1 != Genus_2)
  coringTaxalongParsed2 <- coringTaxalongParsed %>%
    rename(Genus1 = Genus_1, Genus2 = Genus_2)
  return(as.data.frame(coringTaxalongParsed2))
}

# correlationMicrobes<-coringTaxa(microbeAbund)


## SNPs

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  `RegSnp` creates a dataframe of parsed long snp files
#'
#' This internal function takes the orginal snp dataframe and retruns a long parsed snp dataframe
#' @param SnpFile the snp file (rownames is sample number and colnames is the snps)
#' @param microbeAbund the microbial abundance file (rownames is sample number and colnames is
#'                     the microbe)
#' @return A long parsed datframe of snps
#' @keywords snp parsed long
#'
# function to make the long snp format
RegSnp <- function(SnpFile, microbeAbund) {
  SnpFilt <- rownames(microbeAbund)
  SnpAss_filt <- SnpFile[rownames(SnpFile) %in% SnpFilt, ]
  mycor <- cor(cbind(SnpAss_filt, microbeAbund))
  mycor[is.na(mycor)] <- 0
  # Remove initial coloumns for rs correlation
  n_snps <- length(SnpFile)
  n_mycor <- ncol(mycor)
  # Remove the first cor with snps and
  # keep the rest of cor os snps with taxa as it starts there
  mycor_parse <- as.data.frame(mycor[, seq.int(n_snps + 1, n_mycor)]) # Use seqint avoid drama
  # Remove all the lines after rs ends as you do not need the taxa for row
  mycor_parse2 <- as.data.frame(mycor_parse[seq.int(1, n_snps), ])
  mycor_parse2_R2 <- mycor_parse2^2
  mycor_parse2_R2$rsID <- rownames(mycor_parse2_R2)
  mycor_parse2_long <- mycor_parse2_R2 %>%
    pivot_longer(cols = !any_of("rsID"), names_to = "Genus1", values_to = "R2")
  return(mycor_parse2_long)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  `allToAllProduct` creates a dataframe of snp and taxa correlations
#'
#' This internal function takes the original snp dataframe and returns a long parsed snp dataframe
#' @param SnpFile the snp file (rownames is sample number and colnames is the snps)
#' @param microbeAbund he taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)
#' @param rsID Default is NULL and will run across the who dataset unless specific rsID/SNP/chr_region is specified
#' @return A dataframe of correlations between snps and taxa
#' @keywords snptaxa Correlation
#' @export
#' @examples
#' data(microbeAbund)
#' data(SnpFile)
#' x <- allToAllProduct(SnpFile, microbeAbund, "chr1.171282963_T")
#'
# function to make R2
allToAllProduct <- function(SnpFile, microbeAbund, rsID = NULL) {
  stopifnot("data frame is the expected input for SnpFile" = is(SnpFile, "data.frame"))
  stopifnot("data frame is the expected input for microbeAbund" = is(microbeAbund, "data.frame"))
  stopifnot("The variable should match the concordant varible for a specific 'rsID'" = is.null(rsID) || is.character(rsID))
  r2_long <- RegSnp(SnpFile, microbeAbund)
  # Define function apply_product
  # Function to solve on one of the items
  apply_product <- function(rsID) {
    filtered <- filter(r2_long, rsID == !!rsID)
    vapply_type <- array(data = 0.0, dim = length(filtered$R2))
    mtx <- vapply(filtered$R2, `*`, vapply_type, filtered$R2)
    rownames(mtx) <- filtered$Genus1
    colnames(mtx) <- filtered$Genus1
    mtx <- as.data.frame(1.0 / mtx)
    mtx$rsID <- rsID
    mtx$Genus1 <- rownames(mtx)
    result <- pivot_longer(mtx,
      cols = !any_of(c("rsID", "Genus1")),
      names_to = "Genus2", values_to = "product_r2"
    )
    return(result)
  }
  # Get rows for rsID
  if (!is.null(rsID)) { # If value of rsID is not null do this:
    return(apply_product(rsID))
    # Now apply to rest
  } else {
    rsids <- unique(r2_long$rsID)
    result <- lapply(rsids, apply_product) %>%
      bind_rows() %>%
      filter(Genus1 != Genus2)
    return(as.data.frame(result))
  }
}

# one_rs_id<-allToAllProduct(SnpFile,microbeAbund,"chr1.171282963_T")
# for_all_rsids<-allToAllProduct(SnpFile,microbeAbund)


## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  `taxaSnpCor` for estimation of the rho value between snp, taxa correlations across datasets
#'
#' This function produces a log heatmap +1 of the correlation rho values across snp, taxa dataframe.
#' @param for_all_rsids A dataframe result of correlation analysis between the snps and taxa dataframe,
#'  an output of `allToAllProduct()` function.
#' @param correlationMicrobes A dataframe of correlation between `coringTaxa()` function.
#' @param probs Default is NULL if other that all rho values are wanted the value can be subseted using c(`x`,`y`).
#' @return A data frame of correlations between taxa
#' @export
#' @keywords rho estimation
#' @examples
#' data(microbeAbund)
#' data(SnpFile)
#'
#' for_all_rsids <- allToAllProduct(SnpFile, microbeAbund)
#' correlationMicrobes <- coringTaxa(microbeAbund)
#' x <- taxaSnpCor(for_all_rsids, correlationMicrobes)
#'
taxaSnpCor <- function(for_all_rsids, correlationMicrobes, probs = NULL) {
  stopifnot("data frame is the expected input for for_all_rsids" = is(for_all_rsids, "data.frame"))
  stopifnot("data frame is the expected input for correlationMicrobes" = is(correlationMicrobes, "data.frame"))
  stopifnot("The variable for 'probs'" = is.null(probs) || is.numeric(probs))
  invprods_and_cors <- for_all_rsids %>%
    left_join(correlationMicrobes, by = c("Genus1", "Genus2")) %>%
    filter(is.finite(product_r2)) %>%
    mutate(rho_xy = product_r2 * R)
  # Deduplicate gene combinations (i.e., "A", "B" == "B", "A", per rsID)
  invprods_and_cors <- invprods_and_cors %>%
    mutate(
      X1 = ifelse(Genus1 < Genus2, Genus1, Genus2),
      X2 = ifelse(Genus1 > Genus2, Genus1, Genus2)
    )
  invprods_and_cors$duplicated <- duplicated(invprods_and_cors %>% select(rsID, X1, X2))
  invprods_and_cors <- invprods_and_cors %>%
    filter(!duplicated) %>%
    select(-X1, -X2, -duplicated)
  if (is.null(probs)) { # If value of rsID is not null do this:
    return(invprods_and_cors)
    # Now apply to rest
  } else {
    extremes <- quantile(invprods_and_cors$rho_xy, probs = probs) ## ask about this
    final_var <- invprods_and_cors %>% filter(rho_xy < extremes[1] | rho_xy > extremes[2])
    final_var$Genus1 <- gsub("X.", "", final_var$Genus1)
    final_vara <- final_var %>%
      mutate_if(is.character, str_trim)
    final_var_long <- final_vara[final_vara$Genus1 != final_vara$Genus2, ]
    return(final_var_long)
  }
}

# taxa_SNP_Cor<-taxaSnpCor(for_all_rsids,correlationMicrobes)
# taxa_SNP_Cor_lim<-taxaSnpCor(for_all_rsids,correlationMicrobes,probs = c(0.0001, 0.9999))


## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  `mbQtlCorHeatmap` for making heatmap for snp, taxa rho values
#'
#' This function produces a log heatmap +1 of the correlation rho values across snp, taxa datasets
#' @param final_var_long the long data frame of rho values created by the `taxaSnpCor()` function.
#' @param labels_col set to NULL ass default if TRUE, labels will appear on the heatmap.
#' @param ... all other parameters for pheatmap.
#' @return A data frame of correlations between taxa
#' @export
#' @keywords rho heatmap
#' @examples
#'
#' data(microbeAbund)
#' data(SnpFile)
#' for_all_rsids <- allToAllProduct(SnpFile, microbeAbund)
#' correlationMicrobes <- coringTaxa(microbeAbund)
#' taxaSnpCor(for_all_rsids, correlationMicrobes)
#' final_var_long <- taxaSnpCor(for_all_rsids, correlationMicrobes, probs = c(0.0001, 0.9999))
#' x <- mbQtlCorHeatmap(final_var_long)
#'
mbQtlCorHeatmap <- function(final_var_long, labels_col = NULL, ...) {
  stopifnot("data frame is the expected input for final_var_long" = is(final_var_long, "data.frame"))
  stopifnot("TRUE/FALSE for label appearance 'labels_col'" = is.null(labels_col) || is.logical(labels_col))
  final_var_long$Combined_Genus <- paste(final_var_long$Genus1, final_var_long$Genus2, sep = "_")
  final_var2 <- final_var_long %>% select(rsID, Combined_Genus, rho_xy)
  final_var_wide <- final_var2 %>%
    pivot_wider(
      names_from = Combined_Genus, values_from = rho_xy,
      values_fill = 0
    ) %>%
    as.data.frame()
  rownames(final_var_wide) <- final_var_wide$rsID
  final_var_wide2 <- final_var_wide %>% select(-rsID)
  if (is.null(labels_col)) {
    return(pheatmap(as.matrix(log(abs(final_var_wide2) + 1)), ...))
  } else {
    return(pheatmap(as.matrix(log(abs(final_var_wide2) + 1)), labels_col = FALSE, ...))
  }
}
# mbQtlCorHeatmap(taxa_SNP_Cor_lim,fontsize_col=4,fontsize_row=4)

#' @import readxl
#' @import tidyr
#' @import dplyr
#' @import stringr
#' @import metagenomeSeq
#' @importFrom methods is
NULL

#' ## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  metagenomeSeqToMbqtl Converts metagenomeSeq obj to compatible taxa dataframe
#'
#' This function takes and `MRexperiement` class object transforms it and makes the result dataframe compatible
#' with `mbQTL` taxa input file
#' @param meta_glom `MRexperiement` class obj from `metagenomeSeq` package.
#' @param norm A logical indicating whether or not to return normalized counts.
#' @param log TRUE/FALSE whether or not to log2 transform scale.
#' @param aggregate_taxa it is recommended that the normalization occurs at taxa level (default NULL)
#'                       however, if the user chooses to aggregate on the phyla/family/Genus or Species
#'                       level before normalization they have the option.
#' @return A data frame of normalized/not normalized counts compatible with `mbQTL`.
#' @keywords metagenomeSeq MRexperiment normalization
#' @export
#' @examples
#' data(metagenomeSeqObj)
#' x <- metagenomeSeqToMbqtl(metagenomeSeqObj, norm = TRUE, log = TRUE, aggregate_taxa = NULL)
metagenomeSeqToMbqtl <- function(meta_glom, norm, log, aggregate_taxa = NULL) {
  stopifnot("MRexperiment object expected for 'meta_glom'" = is(meta_glom, "MRexperiment"))
  stopifnot("TRUE/FALSE expected for 'norm'" = is.logical(norm))
  stopifnot("TRUE/FALSE expected for 'log'" = is.logical(log))
  stopifnot(
    "agreggate_taxa can only have the following values 'Phylum', 'Family', 'Genus', 'Species'" =
      is.null(aggregate_taxa) || aggregate_taxa %in% c("Phylum", "Family", "Genus", "Species")
  )
  if (!is.null(aggregate_taxa)) {
    if (length(aggregate_taxa) != 1 || !(aggregate_taxa %in% names(meta_glom@featureData@data))) {
      stop("The aggregate_taxa needs to be assigned as NULL or one of these values, Kingdom,
      Phylum,Class,Order,Family,Genus,Species")
    }
  }
  if (is.null(aggregate_taxa)) {
    obj <- meta_glom
    mat <- t(MRcounts(obj, norm = norm, log = log))
    return(as.data.frame(mat))
  } else {
    obj <- metagenomeSeq::aggTax(meta_glom, lvl = aggregate_taxa)
    mat <- t(MRcounts(obj, norm = norm, log = log))
    return(as.data.frame(mat))
  }
}

# compatible_metagenome_seq<-metagenomeSeqToMbqtl(metagenomeSeqObj, norm=TRUE, log=TRUE,
#                                                 aggregate_taxa= "Genus")

#' title `mbQTL` is a package for microbial QTL/GWAS Analysis
#' 
#' @description 
#' 
#' This package provides statistical methods for identifying significant relationships between
#' microbial/taxa and genetic SNP signatures. We use three models 1) linear regression between 
#' all taxa-snp. Main function is `linearTaxaSnp()`. 2) Correlation analysis between taxa-snp
#' across all taxa and snps. Main function is `taxaSnpCor()` and 3) Logistic regression analysis
#' between each taxa and each snp simultaneously or for a specific cases. Main function is
#' `logRegSnpsTaxa()`.
#' 
#' @seealso
#' The package vignette can be accessed with `vignette("mbQTL")`.
#' 
"_PACKAGE"

#' @import ggplot2
#' @import readxl
#' @import tidyr
#' @import dplyr
#' @import stringr
#' @import MatrixEQTL
#' @importFrom graphics hist
#' @importFrom methods is
NULL

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  linearTaxaSnp Performs linear regression analysis between taxa and SNPs and returns concordance statistics
#'
#' This function creates a dataframe output from the results all snps with all taxa linear regression analysis of all
#' snps in the dataset. The result is a dataframe with P values and FDRs of all regressions. MatrixeQTL core functions
#' are utilized to achieve this. Note the main functions used are Matrix_eQTL_engine() assuming linear regression
#' with or without a covariate file.
#' @param microbeAbund the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)
#' @param SnpFile the snp dataframe (values 0,1,2 indicating zygosity), rownames sample names and colnames snp names.
#' @param Covariate default is NULL, hence assumed non-existent. If covariates are available they need to be formatted
#' in the CovFile format, that is colnames are sample numbers matching samples in the microbe abundance and snp file
#' and row names are the co-variates names (such as sex, disease etc).
#' @return A data frame which is a result of Linear Regression of all snp, taxa relationships,
#' with P values and P value corrected values.
#' @export
#' @keywords taxa snp linear regression LR
#' @examples
#' data(microbeAbund)
#' data(SnpFile)
#' data(CovFile)
#' x <- linearTaxaSnp(microbeAbund, SnpFile, Covariate = CovFile)
#'
linearTaxaSnp <- function(microbeAbund, SnpFile, Covariate = NULL) {
  stopifnot("data frame is the expected input for SnpFile" = is(SnpFile, "data.frame"))
  stopifnot("data frame is the expected input for microbeAbund" = is(microbeAbund, "data.frame"))
  stopifnot("data frame is the expected input for Covariate make sure the formatting is correct!" = is.null(Covariate) || is(Covariate, "data.frame"))
  microbeAbund_i <- t(microbeAbund)
  SnpFile_i <- t(SnpFile)
  if (is.null(Covariate)) {
    useModel <- modelLINEAR # modelANOVA or modelLINEAR or modelLINEAR_CROSS
    pvOutputThreshold <- 5e-2
    errorCovariance <- numeric()
    snps <- SlicedData$new()
    SNPs_t_f2 <- as.matrix(SnpFile_i)
    snps$CreateFromMatrix(SNPs_t_f2)
    snps$fileOmitCharacters <- "NA" # denote missing values;
    snps$fileSliceSize <- 2000 # read file in pieces of 2,000 rows
    gene <- SlicedData$new()
    microbe_file_t_f2 <- as.matrix(microbeAbund_i)
    gene$CreateFromMatrix(microbe_file_t_f2)
    gene$fileDelimiter <- "\t" # the TAB character
    gene$fileOmitCharacters <- "NA" # denote missing values;
    gene$fileSliceSize <- 1000 # read file in pieces of 2,000 rows

    outPut_linear <- tempfile()
    # For P value dataframe
    me <- Matrix_eQTL_engine(
      snps = snps,
      gene = gene,
      output_file_name = outPut_linear,
      pvOutputThreshold = pvOutputThreshold,
      useModel = useModel,
      errorCovariance = errorCovariance,
      verbose = TRUE,
      pvalue.hist = TRUE,
      min.pv.by.genesnp = FALSE,
      noFDRsaveMemory = FALSE
    )

    kettle <- as.data.frame(cbind(
      me$all[["eqtls"]][["snps"]], me$all[["eqtls"]][["gene"]],
      me$all[["eqtls"]][["pvalue"]], me$all[["eqtls"]][["FDR"]]
    ))
    names(kettle) <- c("snps", "Genus", "pvalue", "FDR")
    return(kettle)
  } else {
    useModel <- modelLINEAR # modelANOVA or modelLINEAR or modelLINEAR_CROSS
    pvOutputThreshold <- 5e-2
    errorCovariance <- numeric()
    snps <- SlicedData$new()
    SNPs_t_f2 <- as.matrix(SnpFile_i)
    snps$CreateFromMatrix(SNPs_t_f2)
    snps$fileOmitCharacters <- "NA" # denote missing values;
    snps$fileSliceSize <- 2000 # read file in pieces of 2,000 rows

    gene <- SlicedData$new()
    microbe_file_t_f2 <- as.matrix(microbeAbund_i)
    gene$CreateFromMatrix(microbe_file_t_f2)
    gene$fileDelimiter <- "\t" # the TAB character
    gene$fileOmitCharacters <- "NA" # denote missing values;
    gene$fileSliceSize <- 1000 # read file in pieces of 1,000 rows

    cvrt <- SlicedData$new()
    covariate_f <- as.matrix(Covariate)
    cvrt$CreateFromMatrix(covariate_f)
    cvrt$fileDelimiter <- "\t" # the TAB character
    cvrt$fileOmitCharacters <- "NA" # denote missing values;
    cvrt$fileSliceSize <- 10 # read file in pieces of 1,0 rows

    outPut_linear <- tempfile()
    # For P value dataframe
    me <- Matrix_eQTL_engine(
      snps = snps,
      gene = gene,
      cvrt = cvrt,
      output_file_name = outPut_linear,
      pvOutputThreshold = pvOutputThreshold,
      useModel = useModel,
      errorCovariance = errorCovariance,
      verbose = TRUE,
      pvalue.hist = TRUE,
      min.pv.by.genesnp = FALSE,
      noFDRsaveMemory = FALSE
    )
    kettle <- as.data.frame(cbind(
      me$all[["eqtls"]][["snps"]], me$all[["eqtls"]][["gene"]],
      me$all[["eqtls"]][["pvalue"]], me$all[["eqtls"]][["FDR"]]
    ))
    names(kettle) <- c("snps", "Genus", "pvalue", "FDR")
    return(kettle)
  }
}

# LinearAnalysisTaxaSNP<-linearTaxaSnp(microbeAbund,SnpFile,Covariate=CovFile)
# LinearAnalysisTaxaSNP2<-linearTaxaSnp(microbeAbund,SnpFile)


## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  histPvalueLm a histogram of Taxa and SNP linear regression analysis.
#'  This function creates a histogram object of all SNPs with all taxa Linear regression analysis p values.
#' @param LinearAnalysisTaxaSNP the data frame result created from the `linearTaxaSnp()` function.
#' @return A histogram object of p values observed from taxa and SNP Linear Regression analysis.
#' @export
#' @keywords taxa snp linear_regression plot histogram
#' @examples
#' data(microbeAbund)
#' data(microbeAbund)
#' data(SnpFile)
#' data(CovFile)
#' LinearAnalysisTaxaSNPFile <- linearTaxaSnp(microbeAbund, SnpFile, Covariate = CovFile)
#' x <- histPvalueLm(LinearAnalysisTaxaSNPFile)
#'
histPvalueLm <- function(LinearAnalysisTaxaSNP) {
  stopifnot("data frame is the expected input for LinearAnalysisTaxaSNP" = is(LinearAnalysisTaxaSNP, "data.frame"))
  return(hist(as.numeric(LinearAnalysisTaxaSNP$pvalue),
    col = "grey",
    main = "Histogram of Taxa-SNP LM P-Values", xlab = "P value",
    ylab = "Number of Samples"
  ))
}

# histPvalueLm(LinearAnalysisTaxaSNP)


## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, January 2023
#'  qqPlotLm creates QQ-Plot of all SNPs with all taxa Linear regression analysis
#'  This function creates QQ-Plot object of all SNPs with all taxa Linear regression analysis of
#'  expected versus observed P values
#' @param microbeAbund the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)
#' @param SnpFile the snp dataframe (values 0,1,2 indicating zygosity), rownames sample names and colnames snp names.
#' @param Covariate default is NULL, hence assumed non-existent. If covariates are available they need to be formatted
#' in the CovFile format, that is colnames are sample numbers matching samples in the microbe abundance and snp file
#'  and row names are the covariates names (such as sex, disease etc).
#' @return A QQplot object of expected versus obsesrved taxa and SNP Linear Regression analysis
#' @export
#' @keywords taxa snp linear_regression plot
#' @examples
#' data(microbeAbund)
#' data(SnpFile)
#' data(CovFile)
#' x <- qqPlotLm(microbeAbund, SnpFile, Covariate = CovFile)
#'
qqPlotLm <- function(microbeAbund, SnpFile, Covariate = NULL) {
  stopifnot("data frame is the expected input for SnpFile" = is(SnpFile, "data.frame"))
  stopifnot("data frame is the expected input for microbeAbund" = is(microbeAbund, "data.frame"))
  stopifnot("data frame is the expected input for Covariate make sure the formatting is correct!" = is.null(Covariate) || is(Covariate, "data.frame"))
  microbeAbund_i <- t(microbeAbund)
  SnpFile_i <- t(SnpFile)
  if (is.null(Covariate)) {
    # useModel <- modelLINEAR # modelANOVA or modelLINEAR or modelLINEAR_CROSS
    # pvOutputThreshold <- 5e-2
    # errorCovariance <- numeric()
    snps <- SlicedData$new()
    SNPs_t_f2 <- as.matrix(SnpFile_i)
    snps$CreateFromMatrix(SNPs_t_f2)
    snps$fileOmitCharacters <- "NA" # denote missing values;
    snps$fileSliceSize <- 2000 # read file in pieces of 2,000 rows
    gene <- SlicedData$new()
    microbe_file_t_f2 <- as.matrix(microbeAbund_i)
    gene$CreateFromMatrix(microbe_file_t_f2)
    gene$fileDelimiter <- "\t" # the TAB character
    gene$fileOmitCharacters <- "NA" # denote missing values;
    gene$fileSliceSize <- 1000 # read file in pieces of 2,000 rows

    # outPut_linear <- tempfile()
    # For qqPlot
    filename <- tempfile()
    meq <- Matrix_eQTL_engine(
      snps = snps,
      gene = gene,
      # cvrt = cvrt1,
      output_file_name = filename,
      pvOutputThreshold = 1e-6,
      useModel = modelLINEAR,
      errorCovariance = numeric(),
      verbose = TRUE,
      pvalue.hist = "qqplot"
    )
    unlink(filename)
    return(plot(meq, pch = 16, cex = 0.7))
  } else {
    # useModel <- modelLINEAR # modelANOVA or modelLINEAR or modelLINEAR_CROSS
    # pvOutputThreshold <- 5e-2
    # errorCovariance <- numeric()
    snps <- SlicedData$new()
    SNPs_t_f2 <- as.matrix(SnpFile_i)
    snps$CreateFromMatrix(SNPs_t_f2)
    snps$fileOmitCharacters <- "NA" # denote missing values;
    snps$fileSliceSize <- 2000 # read file in pieces of 2,000 rows

    gene <- SlicedData$new()
    microbe_file_t_f2 <- as.matrix(microbeAbund_i)
    gene$CreateFromMatrix(microbe_file_t_f2)
    gene$fileDelimiter <- "\t" # the TAB character
    gene$fileOmitCharacters <- "NA" # denote missing values;
    gene$fileSliceSize <- 1000 # read file in pieces of 1,000 rows

    cvrt <- SlicedData$new()
    # covariate_f <- as.matrix(Covariate)
    cvrt$fileDelimiter <- "\t" # the TAB character
    cvrt$fileOmitCharacters <- "NA" # denote missing values;
    cvrt$fileSliceSize <- 10 # read file in pieces of 1,0 rows

    filename <- tempfile()
    meq <- Matrix_eQTL_engine(
      snps = snps,
      gene = gene,
      # cvrt = cvrt1,
      output_file_name = filename,
      pvOutputThreshold = 1e-6,
      useModel = modelLINEAR,
      errorCovariance = numeric(),
      verbose = TRUE,
      pvalue.hist = "qqplot"
    )
    unlink(filename)
    return(plot(meq, pch = 16, cex = 0.7))
  }
}


# qqPlotLm(microbeAbund,SnpFile,Covariate=CovFile)

#' `mbQTL` "CovFile"
#'
#' The "CovFile" is the covariate file for linear regression option of taxa and snp association.
#' The covariance file is generated randomly by assigning sex and site of collection to
#' the samples.) rownames are covariate and colnames samples.
#' @name CovFile
#' @docType data
#' @keywords data
NULL

#' `mbQTL` "SnpFile"
#' 
#' The "SnpFile" is randomly generated from GATK (Van der Auwera GA & O'Connor BD. (2020).
#' Genomics in the Cloud: Using Docker, GATK, and WDL in Terra (1st Edition).
#' O'Reilly Media) snp calls followed by plink (Purcell S, et al. (2007) PLINK: a
#' toolset for whole-genome association and population-based linkage analysis. American
#' Journal of Human Genetics) processing and it needs to be in (0,1,2) format representing
#' the zygosity of the snps.
#'
#' @name SnpFile
#' @docType data
#' @keywords data
NULL

#'  `mbQTL` "metagenomeSeqObj"
#'  
#' "MetagenomSeqObj" is an `MRexperiment` object format of the "microbeAbund" file.
#'
#' @name metagenomeSeqObj
#' @docType data
#' @keywords data
NULL

#' `mbQTL` "microbiomeAbund" File
#' 
#' This is the microbial Abudnance file generated from 16s it is either this file or
#' the "metaGenomeSeqObj".The "microbiomeAbund" file is a randomly generated file
#' format for total microbe presence (number of reads)(parasite/viral transcripts)
#' for specific species.This could be generated from select taxa results from 
#' `biom()` or `MRexperiment` objects as long as the samples are colnames and taxa
#'  are rownames.
#'
#' @name microbeAbund
#' @docType data
#' @keywords data
NULL

---
title: "mbQTL_Vignette"
date: "2023-02-05"
author: "Mercedeh Movassagh"
package: mbQTL
output:
  BiocStyle::html_document:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{MbQTL_Vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r style, echo=FALSE, results='asis'}
BiocStyle::markdown()
```


```{css, echo=FALSE}
.header-section-number {
  display: none;
}

body {
  counter-reset: counter-level-1;
}

h1:not(.title) {
  counter-increment: counter-level-1;
  counter-reset: counter-level-2;
}

h1:not(.title)::before{
  content: counter(counter-level-1, upper-alpha) ". ";
}

h2 {
  counter-incrementx: counter-level-2;
  counter-reset: counter-level-3;
}

h2::before {
  content: counter(counter-level-1, upper-alpha) "." counter(counter-level-2) " ";
}

h3 {
  counter-increment: counter-level-3;
}

h3::before {
  content: counter(counter-level-1, upper-alpha) "." counter(counter-level-2) "." counter(counter-level-3) " ";
}
```

```{r setup}
knitr::opts_chunk$set(
  dpi = 300,
  warning = FALSE,
  collapse = TRUE,
  error = FALSE,
  comment = "#>",
  echo = TRUE
)
library(mbQTL)
library(tidyr)

data("SnpFile")
data("microbeAbund")
data("CovFile")
data("metagenomeSeqObj")

doctype <- knitr::opts_knit$get("rmarkdown.pandoc.to")
```

```{r}
sessionInfo()
```

# Introduction

`mbQTL` is a statistical package for identifying significant taxa (microbial), 
snp (genotype) relationships across large sample sets. The package measures 
this relationship using three different approaches. A) linear regression analysis
which uses core `MatrixeQTL` model. B) rho estimation for correlation of taxa and SNP,
which is a novel method of estimating rho values after R value of taxa-taxa is 
estimated then adjusted for association of taxa-SNP. Using this method various
taxa-SNP associations can be estimated simultaneously, and linkage disequilibrium (LD)
snps and regions can be identified in terms of presence and their associations with
snps (see section for more details). C) Utilizing Logistic regression analysis, we
estimate the relationship of one snp and one taxa across the sample cohort but
simultaneously this could be estimated for all samples and proper statistics is
produced from these regressions. Appropriate plots are generated for all three
methods utilized in this vignette and are part of the package to simplify 
visualization.

# Data upload.


The datasets used for this vignette are all simulated and are not factual. The "Taxa_table_MVP_cor.txt"
and "SNP_table_MVP_cor.txt" are used in all three sections mentioned above and the "covariates.txt" 
file is used for section A)linear regression. The input file format for "SNP_table_MVP_cor.txt" 
should be colnames with SNP names/locations/rs_numbers and rownames needs to be sample number.
For the "Taxa_table_MVP_cor.txt" the colnames is the microbe name and the row name is the sample
number and finally for the "covariates.txt" file the colnames need to be samplenames and the 
rownames need to be the covariates. Please ensure all 3 files are matching in terms of sample
number and are appropriately formatted to avoid errors in your analysis.

## Import data
mbQTL accepts data in dataframe, `MRexperiment` objects or table formats. The files 
are accessible when the `mbQTL` library is loaded. We have also created a larger simulated 
data set of 500 SNPs and 500 pathogens for 500 patients which can be accessed and 
downloaded from https://doi.org/10.18129/B9.bioc.mbQTL.
The whole data set can be run with an average memory of 90MB in less than 2 
minutes for all models.
The input files are microbial abundance file (rownames taxa, colnames sample 
names) and SnpFile (rownames snps and colnames sample names) with optional 
"CovFile"(rownames covariates and colnames sample names) ( note all samples should
be concordant in the three data sets). Note, the data can be imported as
an R object using either upload() or if RDS file, readRDS(). If the the
the data is a data frame in text format it can be uploaded using read.table(),
in a tab separated manner, if excel, the user can use the read_excel() 
function from readxl package. Alternatively, read.csv() can be used for
import of a csv file.

```{r eval=TRUE, echo=TRUE}
# microbial count file
# microbeAbund
# SNP file
# SnpFile
# Covariance file
# CovFile

#Check out file formats compatible with mbQTL files.

#head(microbeAbund)
#head(SnpFile)
#head(CovFile)

metagenomeSeqObj

```


If the user has an MRexperiment microbiome file (`metagenomeSeq` formatted), one can
use the following function to normalize their `metagenomeSeq` object or aggregate based
on various taxonomic levels before normalization. 
The output will be formatted in compatible data frame format for `mbQTL`. If one has
a `phyloseq` S4 object, they can use the `phyloseq_to_metagenomeSeq()` option of the
`phyloseq` package and start from there.


```{r eval=TRUE, echo=TRUE}
compatible_metagenome_seq <- metagenomeSeqToMbqtl(metagenomeSeqObj,
  norm = TRUE, log = TRUE,
  aggregate_taxa = "Genus"
)

# Check for the class of compatible_metagenome_seq

class(compatible_metagenome_seq)

```


# Part A. Linear regression

Linear regression analysis to identify significant association between taxa and 
SNPs across large sample sized. P values FDRs, Pvalue histogram plots and QQ-plots
are provided for various taxa and SNP regression analysis performed. microbeAbund
is the taxa abundance file, SnpFile us file across samples (0,1,2  (reference,
heterozygous and alternate genotype). `MatrixeQTL` makes its estimations assuming
three different models of which we employ two, a) assumption of simple linear
regression for each gene and SNV pair The first approach is a very similar approach
to that of the expression quantitative trait loci association (eQTL). b) regression
model with covariates (Shabalin, 2012).

Note: The main assumption behind this model is, Abundance of taxa association with
presence of SNV/SNP across cohorts of samples.

```{r eval=TRUE, echo=TRUE}

# perform linear regression analysis between taxa and snps across 
# samples and produce a data frame of results.(covariate file is
# optional but highly recommended to avoid results which might be
# prone to batch effects and obtaining optimal biological relevance
# for finding snp - taxa relationships.) 

LinearAnalysisTaxaSNP <- linearTaxaSnp(microbeAbund,
  SnpFile,
  Covariate = CovFile
)
```


```{r eval=TRUE, echo=TRUE, fig.width = 4, fig.height=4, out.width="90%", dpi=300, fig.align="center"}

# Create a histogram of P values across all snp - taxa
# linear regression estimations

histPvalueLm(LinearAnalysisTaxaSNP)


# Create a QQPlot of expected versus estimated P value for all all
# snp - taxa linear regression estimations

qqPlotLm(microbeAbund, SnpFile, Covariate = CovFile)
```


# Part B. Correlation analysis (rho estimation)

Estimate taxa-taxa correlation (microbeAbund (taxa abundance file)) and estimate R 
(Strength of the relationship between taxa `x` and `y`). In other words we estimate 
Goodness of fits, as a measure of a feature’s association with host genetics 
(Significant SNPs) association with taxa (matching 16S results). In details we
measure the strength of relationship between taxa `x` and taxa `y` (R correlation value),
next we measure goodness of fits R2 and ask to what extent the variance in one taxa
explains the variance in a SNP,  and finally we find clusters of taxa associated with
various SNPs across the dataset by evaluating and controlling for both these measures
together and finally estimating rho value.

Note: The main idea behind this approach estimating strength of a correlation 
between SNP-Taxa and Taxa-Taxa ( communities of bacteria/organisms).


```{r eval=TRUE, echo=TRUE}

# Estimate taxa SNP - taxa correlation R and estimate R2
# (To what extent variance in one taxa explains the
# variance in snp)

# First estimate R value for all SNP-Taxa relationships

correlationMicrobes <- coringTaxa(microbeAbund)

# Estimate the correlation value (R2) between a specific snp and all taxa.

one_rs_id <- allToAllProduct(SnpFile, microbeAbund, "chr1.171282963_T")

# Estimate the correlation value (R2) between all snps and all taxa.

for_all_rsids <- allToAllProduct(SnpFile, microbeAbund)

# Estimate rho value for all snps and all taxa present in the dataset.

taxa_SNP_Cor <- taxaSnpCor(for_all_rsids, correlationMicrobes)

# Estimate rho value for all snps and all taxa present in the
# dataset and pick the highest and lowest p values in the
# dataset to identify improtant SNP-Taxa relationships.

taxa_SNP_Cor_lim <- taxaSnpCor(for_all_rsids,
  correlationMicrobes,
  probs = c(0.0001, 0.9999)
)
```


```{r eval=TRUE, echo=TRUE, fig.width = 9, fig.height=5, out.width="90%", dpi=300, fig.align="center"}

## Draw heatmap of rho estimates "taxa_SNP_Cor_lim" is the
# output of taxaSnpCor().
# One can use other pheatmap() settings for extra annotation

mbQtlCorHeatmap(taxa_SNP_Cor_lim,
  fontsize_col = 5,
  fontsize_row = 7
)
```


# Part C. Logistic Regression analysis
SNPs and then Genus/species (categorical(presence/absence)) as expression.
For this we need to binarize out taxa abundance file based on a cutoff to a 
zero or one or presence or absence format. We assume a presence and absence 
binary/categorical relationship between every taxa and SNP pair and use a 
logistic regression model to identify significance taxa-SNP relationships.

Note: The main hypothesis behind this approach is there an association between
presence of taxa (present/absent) and SNV/SNP.

```{r eval=TRUE, echo=TRUE}

# perform Logistic regression analysis between taxa and snps 
# across samples microbeAbund is the microbe abundance file
# (a dateframe with rownames as taxa and colnames and sample
# names) and SnpFile is the snp file (0,1,2) values
# (rownames as snps and colnames as samples)

log_link_resA <- logRegSnpsTaxa(microbeAbund, SnpFile)

# Perform Logistic regression for specific microbe

log_link_resB <- logRegSnpsTaxa(microbeAbund, SnpFile,
  selectmicrobe = c("Haemophilus")
)
```



```{r eval=TRUE, echo=TRUE, fig.width = 5, fig.height=2, out.width="90%", dpi=300, fig.align="center"}

# Create a barplot with the specific rsID, and microbe of interest,
# including the genotype information for the reference versus 
# alternate versus hetrozygous alleles and and presence absence
# of microbe of interest. Note your reference, alternative and
# heterozygous genotype values need to match the genotype of
# your SNP of interest this information can be obtained
# from snpdb data base or GATK/plink files.

Logit_plot <- logitPlotSnpTaxa(microbeAbund, SnpFile,
  selectmicrobe = "Neisseria",
  rsID = "chr2.241072116_A", ref = "GG", alt = "AA", het = "AG"
)

Logit_plot
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{mbQTL-package}
\alias{mbQTL}
\alias{mbQTL-package}
\title{title \code{mbQTL} is a package for microbial QTL/GWAS Analysis}
\description{
This package provides statistical methods for identifying significant relationships between
microbial/taxa and genetic SNP signatures. We use three models 1) linear regression between
all taxa-snp. Main function is \code{linearTaxaSnp()}. 2) Correlation analysis between taxa-snp
across all taxa and snps. Main function is \code{taxaSnpCor()} and 3) Logistic regression analysis
between each taxa and each snp simultaneously or for a specific cases. Main function is
\code{logRegSnpsTaxa()}.
}
\seealso{
The package vignette can be accessed with \code{vignette("mbQTL")}.
}
\author{
\strong{Maintainer}: Mercedeh Movassagh \email{mercedeh.movassagh@yale.edu} (\href{https://orcid.org/0000-0001-7690-0230}{ORCID})

Authors:
\itemize{
  \item Steven Schiff
  \item Joseph N Paulson
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Correlation_Regression.R
\name{taxaSnpCor}
\alias{taxaSnpCor}
\title{\code{taxaSnpCor} for estimation of the rho value between snp, taxa correlations across datasets}
\usage{
taxaSnpCor(for_all_rsids, correlationMicrobes, probs = NULL)
}
\arguments{
\item{for_all_rsids}{A dataframe result of correlation analysis between the snps and taxa dataframe,
an output of \code{allToAllProduct()} function.}

\item{correlationMicrobes}{A dataframe of correlation between \code{coringTaxa()} function.}

\item{probs}{Default is NULL if other that all rho values are wanted the value can be subseted using c(\code{x},\code{y}).}
}
\value{
A data frame of correlations between taxa
}
\description{
This function produces a log heatmap +1 of the correlation rho values across snp, taxa dataframe.
}
\examples{
data(microbeAbund)
data(SnpFile)

for_all_rsids <- allToAllProduct(SnpFile, microbeAbund)
correlationMicrobes <- coringTaxa(microbeAbund)
x <- taxaSnpCor(for_all_rsids, correlationMicrobes)

}
\keyword{estimation}
\keyword{rho}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Logisitic_Regression.R
\name{binarizeMicrobe}
\alias{binarizeMicrobe}
\title{\code{binarizeMicrobe} binarizes microbe abundace file based on user's cutoff}
\usage{
binarizeMicrobe(microbeAbund, cutoff = NULL, selectmicrobe = NULL)
}
\arguments{
\item{microbeAbund}{the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)}

\item{cutoff}{cutoff at which the user chose to call taxa positive or negative across samples (should be a numeric value
for normalized or count values).}

\item{selectmicrobe}{default is and all taxa are considered at the same time, if the user is interested in a
specific pathogen use name of the pathogen for example "Haemophilus".}
}
\value{
A data frame of microbial abundance.
}
\description{
This function creates a dataframe output produces a formatted dataframe prepared.
}
\keyword{logitParsing}
\keyword{logitPlotDataframe}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Logisitic_Regression.R
\name{prepareCorData}
\alias{prepareCorData}
\title{\code{prepareCorData} prpares and joins snp-taxa and taxa-taxa correlation file.}
\usage{
prepareCorData(microbeAbund, SnpFile, cutoff = NULL, selectmicrobe = NULL)
}
\arguments{
\item{microbeAbund}{the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)}

\item{SnpFile}{the snp dataframe (values 0,1,2 indicating zygosity), rownames sample names and colnames snp names.}

\item{cutoff}{default is NULL, hence anything above cutoff is considered positive, the cutoff at which the
specific or all taxa are considered positive for the pathogen (1 indicates positive and 0 negative).}

\item{selectmicrobe}{default is and all taxa are considered at the same time, if the user is interested in a
specific pathogen use name of the pathogen for example "Haemophilus".}
}
\value{
A data frame which is a result of Logistic regression  products of individual snp, taxa relationships,
with P values and P value corrected values.
}
\description{
This function creates a dataframe output produces a formatted dataframe prepared.
}
\keyword{logitPlotDataframe}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Correlation_Regression.R
\name{RegSnp}
\alias{RegSnp}
\title{\code{RegSnp} creates a dataframe of parsed long snp files}
\usage{
RegSnp(SnpFile, microbeAbund)
}
\arguments{
\item{SnpFile}{the snp file (rownames is sample number and colnames is the snps)}

\item{microbeAbund}{the microbial abundance file (rownames is sample number and colnames is
the microbe)}
}
\value{
A long parsed datframe of snps
}
\description{
This internal function takes the orginal snp dataframe and retruns a long parsed snp dataframe
}
\keyword{long}
\keyword{parsed}
\keyword{snp}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Correlation_Regression.R
\name{coringTaxa}
\alias{coringTaxa}
\title{\code{coringTaxa} creates correlation dataframe for taxa}
\usage{
coringTaxa(microbeAbund)
}
\arguments{
\item{microbeAbund}{the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)}
}
\value{
A data frame of correlations between taxa
}
\description{
This function creates an output correlation data frame for all microbial taxa (or other organisms such as
viral or parasitic taxa)
}
\examples{
data(microbeAbund)
x <- coringTaxa(microbeAbund)

}
\keyword{correlation}
\keyword{taxa}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SnpFile}
\alias{SnpFile}
\title{\code{mbQTL} "SnpFile"}
\description{
The "SnpFile" is randomly generated from GATK (Van der Auwera GA & O'Connor BD. (2020).
Genomics in the Cloud: Using Docker, GATK, and WDL in Terra (1st Edition).
O'Reilly Media) snp calls followed by plink (Purcell S, et al. (2007) PLINK: a
toolset for whole-genome association and population-based linkage analysis. American
Journal of Human Genetics) processing and it needs to be in (0,1,2) format representing
the zygosity of the snps.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Logisitic_Regression.R
\name{logitPlotSnpTaxa}
\alias{logitPlotSnpTaxa}
\title{logitPlotSnpTaxa produces bar plots for counts of ref vs alt vs het allells for
particular rsID taxa combinations}
\usage{
logitPlotSnpTaxa(
  microbeAbund,
  SnpFile,
  selectmicrobe = NULL,
  rsID,
  ref = NULL,
  alt = NULL,
  het = NULL,
  color = NULL,
  cutoff = NULL
)
}
\arguments{
\item{microbeAbund}{original microbe abundance file (colnames microbe, rownames= sample IDs)}

\item{SnpFile}{original snp file with (0,1,2 values for ref, het, alt genotypes), colnames
SNP names, rownames, sample IDs.}

\item{selectmicrobe}{name of the microbe of interest (for example individual significant microbes
associate with a snp).}

\item{rsID}{name of the snp of interest (for example individual significant snps associated with
a microbe)}

\item{ref}{the name of reference genotype for example "GG"}

\item{alt}{the name of snp (variant) genotype for example "AA"}

\item{het}{the name of hetrozygote genotype for example "GA"}

\item{color}{the default is NULL and the color is set to c("#ffaa1e", "#87365b").}

\item{cutoff}{cutoff at which we call microbe present or absent}
}
\value{
A bar ggplot comparing the counts of ref vs alt vs het genotype
}
\description{
This function creates a dataframe output produces a formatted dataframe prepared.
}
\examples{
data(microbeAbund)
data(SnpFile)
x <- logitPlotSnpTaxa(microbeAbund, SnpFile,
  selectmicrobe = "Neisseria", rsID = "chr2.241072116_A",
  ref = NULL, alt = NULL, het = NULL, color = NULL, cutoff = NULL
)
}
\keyword{barplot}
\keyword{logitplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{microbeAbund}
\alias{microbeAbund}
\title{\code{mbQTL} "microbiomeAbund" File}
\description{
This is the microbial Abudnance file generated from 16s it is either this file or
the "metaGenomeSeqObj".The "microbiomeAbund" file is a randomly generated file
format for total microbe presence (number of reads)(parasite/viral transcripts)
for specific species.This could be generated from select taxa results from
\code{biom()} or \code{MRexperiment} objects as long as the samples are colnames and taxa
are rownames.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{CovFile}
\alias{CovFile}
\title{\code{mbQTL} "CovFile"}
\description{
The "CovFile" is the covariate file for linear regression option of taxa and snp association.
The covariance file is generated randomly by assigning sex and site of collection to
the samples.) rownames are covariate and colnames samples.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Linear_regression.R
\name{qqPlotLm}
\alias{qqPlotLm}
\title{qqPlotLm creates QQ-Plot of all SNPs with all taxa Linear regression analysis
This function creates QQ-Plot object of all SNPs with all taxa Linear regression analysis of
expected versus observed P values}
\usage{
qqPlotLm(microbeAbund, SnpFile, Covariate = NULL)
}
\arguments{
\item{microbeAbund}{the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)}

\item{SnpFile}{the snp dataframe (values 0,1,2 indicating zygosity), rownames sample names and colnames snp names.}

\item{Covariate}{default is NULL, hence assumed non-existent. If covariates are available they need to be formatted
in the CovFile format, that is colnames are sample numbers matching samples in the microbe abundance and snp file
and row names are the covariates names (such as sex, disease etc).}
}
\value{
A QQplot object of expected versus obsesrved taxa and SNP Linear Regression analysis
}
\description{
qqPlotLm creates QQ-Plot of all SNPs with all taxa Linear regression analysis
This function creates QQ-Plot object of all SNPs with all taxa Linear regression analysis of
expected versus observed P values
}
\examples{
data(microbeAbund)
data(SnpFile)
data(CovFile)
x <- qqPlotLm(microbeAbund, SnpFile, Covariate = CovFile)

}
\keyword{linear_regression}
\keyword{plot}
\keyword{snp}
\keyword{taxa}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Linear_regression.R
\name{linearTaxaSnp}
\alias{linearTaxaSnp}
\title{linearTaxaSnp Performs linear regression analysis between taxa and SNPs and returns concordance statistics}
\usage{
linearTaxaSnp(microbeAbund, SnpFile, Covariate = NULL)
}
\arguments{
\item{microbeAbund}{the taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)}

\item{SnpFile}{the snp dataframe (values 0,1,2 indicating zygosity), rownames sample names and colnames snp names.}

\item{Covariate}{default is NULL, hence assumed non-existent. If covariates are available they need to be formatted
in the CovFile format, that is colnames are sample numbers matching samples in the microbe abundance and snp file
and row names are the co-variates names (such as sex, disease etc).}
}
\value{
A data frame which is a result of Linear Regression of all snp, taxa relationships,
with P values and P value corrected values.
}
\description{
This function creates a dataframe output from the results all snps with all taxa linear regression analysis of all
snps in the dataset. The result is a dataframe with P values and FDRs of all regressions. MatrixeQTL core functions
are utilized to achieve this. Note the main functions used are Matrix_eQTL_engine() assuming linear regression
with or without a covariate file.
}
\examples{
data(microbeAbund)
data(SnpFile)
data(CovFile)
x <- linearTaxaSnp(microbeAbund, SnpFile, Covariate = CovFile)

}
\keyword{LR}
\keyword{linear}
\keyword{regression}
\keyword{snp}
\keyword{taxa}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Linear_regression.R
\name{histPvalueLm}
\alias{histPvalueLm}
\title{histPvalueLm a histogram of Taxa and SNP linear regression analysis.
This function creates a histogram object of all SNPs with all taxa Linear regression analysis p values.}
\usage{
histPvalueLm(LinearAnalysisTaxaSNP)
}
\arguments{
\item{LinearAnalysisTaxaSNP}{the data frame result created from the \code{linearTaxaSnp()} function.}
}
\value{
A histogram object of p values observed from taxa and SNP Linear Regression analysis.
}
\description{
histPvalueLm a histogram of Taxa and SNP linear regression analysis.
This function creates a histogram object of all SNPs with all taxa Linear regression analysis p values.
}
\examples{
data(microbeAbund)
data(microbeAbund)
data(SnpFile)
data(CovFile)
LinearAnalysisTaxaSNPFile <- linearTaxaSnp(microbeAbund, SnpFile, Covariate = CovFile)
x <- histPvalueLm(LinearAnalysisTaxaSNPFile)

}
\keyword{histogram}
\keyword{linear_regression}
\keyword{plot}
\keyword{snp}
\keyword{taxa}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Correlation_Regression.R
\name{allToAllProduct}
\alias{allToAllProduct}
\title{\code{allToAllProduct} creates a dataframe of snp and taxa correlations}
\usage{
allToAllProduct(SnpFile, microbeAbund, rsID = NULL)
}
\arguments{
\item{SnpFile}{the snp file (rownames is sample number and colnames is the snps)}

\item{microbeAbund}{he taxa abundance dataframe (rownames sample names and colnames taxa Genus/species/family)}

\item{rsID}{Default is NULL and will run across the who dataset unless specific rsID/SNP/chr_region is specified}
}
\value{
A dataframe of correlations between snps and taxa
}
\description{
This internal function takes the original snp dataframe and returns a long parsed snp dataframe
}
\examples{
data(microbeAbund)
data(SnpFile)
x <- allToAllProduct(SnpFile, microbeAbund, "chr1.171282963_T")

}
\keyword{Correlation}
\keyword{snptaxa}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Correlation_Regression.R
\name{mbQtlCorHeatmap}
\alias{mbQtlCorHeatmap}
\title{\code{mbQtlCorHeatmap} for making heatmap for snp, taxa rho values}
\usage{
mbQtlCorHeatmap(final_var_long, labels_col = NULL, ...)
}
\arguments{
\item{final_var_long}{the long data frame of rho values created by the \code{taxaSnpCor()} function.}

\item{labels_col}{set to NULL ass default if TRUE, labels will appear on the heatmap.}

\item{...}{all other parameters for pheatmap.}
}
\value{
A data frame of correlations between taxa
}
\description{
This function produces a log heatmap +1 of the correlation rho values across snp, taxa datasets
}
\examples{

data(microbeAbund)
data(SnpFile)
for_all_rsids <- allToAllProduct(SnpFile, microbeAbund)
correlationMicrobes <- coringTaxa(microbeAbund)
taxaSnpCor(for_all_rsids, correlationMicrobes)
final_var_long <- taxaSnpCor(for_all_rsids, correlationMicrobes, probs = c(0.0001, 0.9999))
x <- mbQtlCorHeatmap(final_var_long)

}
\keyword{heatmap}
\keyword{rho}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{metagenomeSeqObj}
\alias{metagenomeSeqObj}
\title{\code{mbQTL} "metagenomeSeqObj"

"MetagenomSeqObj" is an \code{MRexperiment} object format of the "microbeAbund" file.}
\description{
\code{mbQTL} "metagenomeSeqObj"

"MetagenomSeqObj" is an \code{MRexperiment} object format of the "microbeAbund" file.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MetagenomeSeqconverter.R
\name{metagenomeSeqToMbqtl}
\alias{metagenomeSeqToMbqtl}
\title{\subsection{Written by Mercedeh Movassagh \href{mailto:mercedeh@ds.dfci.harvard.edu}{mercedeh@ds.dfci.harvard.edu}, January 2023}{

metagenomeSeqToMbqtl Converts metagenomeSeq obj to compatible taxa dataframe
}}
\usage{
metagenomeSeqToMbqtl(meta_glom, norm, log, aggregate_taxa = NULL)
}
\arguments{
\item{meta_glom}{\code{MRexperiement} class obj from \code{metagenomeSeq} package.}

\item{norm}{A logical indicating whether or not to return normalized counts.}

\item{log}{TRUE/FALSE whether or not to log2 transform scale.}

\item{aggregate_taxa}{it is recommended that the normalization occurs at taxa level (default NULL)
however, if the user chooses to aggregate on the phyla/family/Genus or Species
level before normalization they have the option.}
}
\value{
A data frame of normalized/not normalized counts compatible with \code{mbQTL}.
}
\description{
This function takes and \code{MRexperiement} class object transforms it and makes the result dataframe compatible
with \code{mbQTL} taxa input file
}
\examples{
data(metagenomeSeqObj)
x <- metagenomeSeqToMbqtl(metagenomeSeqObj, norm = TRUE, log = TRUE, aggregate_taxa = NULL)
}
\keyword{MRexperiment}
\keyword{metagenomeSeq}
\keyword{normalization}
